@@ -25,43 +25,12 @@
 #include <stdlib.h>
 #include <glib/gi18n-lib.h>
 
-#include "devhelp.h"
+#include "dh-assistant.h"
 #include "dh-preferences.h"
+#include "dh-window.h"
 
-typedef struct {
-        DhBookManager *book_manager;
-} DhAppPrivate;
+G_DEFINE_TYPE (DhApp, dh_app, GTK_TYPE_APPLICATION);
 
-G_DEFINE_TYPE_WITH_PRIVATE (DhApp, dh_app, GTK_TYPE_APPLICATION);
-
-/**
- * dh_app_peek_book_manager:
- * @app: a #DhApp object
- *
- * Get the associated #DhBookManager.
- *
- * Returns: (transfer none): the book manager associated with this
- */
-DhBookManager *
-dh_app_peek_book_manager (DhApp *app)
-{
-        DhAppPrivate *priv;
-
-        g_return_val_if_fail (DH_IS_APP (app), NULL);
-
-        priv = dh_app_get_instance_private (app);
-
-        return priv->book_manager;
-}
-
-/**
- * dh_app_peek_first_window:
- * @app: a #DhApp object
- *
- * Get the first #DhWindow.
- *
- * Returns: (transfer none): the first window
- */
 GtkWindow *
 dh_app_peek_first_window (DhApp *app)
 {
@@ -84,14 +53,6 @@ dh_app_peek_first_window (DhApp *app)
         return dh_app_peek_first_window (app);
 }
 
-/**
- * dh_app_peek_assistant:
- * @app: a #DhApp object
- *
- * Get the associated #DhAssistant.
- *
- * Returns: (transfer none): the assistant
- */
 GtkWindow *
 dh_app_peek_assistant (DhApp *app)
 {
@@ -136,12 +97,6 @@ _dh_app_has_app_menu (DhApp *app)
 /******************************************************************************/
 /* Application action activators */
 
-/**
- * dh_app_new_window:
- * @app: a #DhApp object
- *
- * Create a new #DhWindow.
- */
 void
 dh_app_new_window (DhApp *app)
 {
@@ -150,12 +105,6 @@ dh_app_new_window (DhApp *app)
         g_action_group_activate_action (G_ACTION_GROUP (app), "new-window", NULL);
 }
 
-/**
- * dh_app_quit:
- * @app: a #DhApp object
- *
- * Quit the application.
- */
 void
 dh_app_quit (DhApp *app)
 {
@@ -164,13 +113,6 @@ dh_app_quit (DhApp *app)
         g_action_group_activate_action (G_ACTION_GROUP (app), "quit", NULL);
 }
 
-/**
- * dh_app_search:
- * @app: a #DhApp object
- * @keyword: the search request
- *
- * Search for @keyword in the entire application.
- */
 void
 dh_app_search (DhApp *app,
                const gchar *keyword)
@@ -180,13 +122,6 @@ dh_app_search (DhApp *app,
         g_action_group_activate_action (G_ACTION_GROUP (app), "search", g_variant_new_string (keyword));
 }
 
-/**
- * dh_app_search_assistant:
- * @app: a #DhApp object
- * @keyword: the search request
- *
- * Search for @keyword in the entire application with a #DhAssistant.
- */
 void
 dh_app_search_assistant (DhApp *app,
                          const gchar *keyword)
@@ -196,12 +131,6 @@ dh_app_search_assistant (DhApp *app,
         g_action_group_activate_action (G_ACTION_GROUP (app), "search-assistant", g_variant_new_string (keyword));
 }
 
-/**
- * dh_app_raise:
- * @app: a #DhApp object
- *
- * Present the main window of the application.
- */
 void
 dh_app_raise (DhApp *app)
 {
@@ -302,7 +231,7 @@ about_cb (GSimpleAction *action,
         gtk_show_about_dialog (parent,
                                "name", _("Devhelp"),
                                "version", PACKAGE_VERSION,
-                               "comments", _("A developers' help browser for GNOME"),
+                               "comments", _("A developers’ help browser for GNOME"),
                                "authors", authors,
                                "translator-credits", _("translator-credits"),
                                "website", PACKAGE_URL,
@@ -469,7 +398,6 @@ static void
 dh_app_startup (GApplication *application)
 {
         DhApp *app = DH_APP (application);
-        DhAppPrivate *priv = dh_app_get_instance_private (app);
 
         g_application_set_resource_base_path (application, "/org/gnome/devhelp");
 
@@ -506,10 +434,6 @@ dh_app_startup (GApplication *application)
 
         /* Setup accelerators */
         setup_accelerators (app);
-
-        /* Load the book manager */
-        priv->book_manager = dh_book_manager_new ();
-        dh_book_manager_populate (priv->book_manager);
 }
 
 static void
@@ -520,13 +444,6 @@ dh_app_activate (GApplication *application)
 
 /******************************************************************************/
 
-/**
- * dh_app_new:
- *
- * Create a new #DhApp object.
- *
- * Returns: a new #DhApp object
- */
 DhApp *
 dh_app_new (void)
 {
@@ -623,27 +540,13 @@ dh_app_init (DhApp *app)
         g_application_add_main_option_entries (G_APPLICATION (app), options);
 }
 
-static void
-dh_app_dispose (GObject *object)
-{
-        DhApp *app = DH_APP (object);
-        DhAppPrivate *priv = dh_app_get_instance_private (app);
-
-        g_clear_object (&priv->book_manager);
-
-        G_OBJECT_CLASS (dh_app_parent_class)->dispose (object);
-}
-
 static void
 dh_app_class_init (DhAppClass *klass)
 {
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);
         GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
         application_class->startup = dh_app_startup;
         application_class->activate = dh_app_activate;
         application_class->handle_local_options = dh_app_handle_local_options;
         application_class->command_line = dh_app_command_line;
-
-        object_class->dispose = dh_app_dispose;
 }
