@@ -25,38 +25,48 @@
 #include <glib/gi18n-lib.h>
 #include "dh-util.h"
 #include "dh-book.h"
+#include "dh-book-manager.h"
+
+/**
+ * SECTION:dh-assistant-view
+ * @Title: DhAssistantView
+ * @Short_description: A small “assistant” widget for displaying just one hit
+ *
+ * #DhAssistantView is a subclass of #WebKitWebView for displaying the
+ * documentation of just one symbol.
+ *
+ * A possible use-case: in a text editor, pressing a keyboard shortcut could
+ * display this widget for the symbol under the cursor.
+ *
+ * With the Devhelp application, an assistant can easily be launched with the
+ * command line option `--search-assistant`.
+ */
 
 typedef struct {
-        DhBookManager *book_manager;
-        DhLink        *link;
-        gchar         *current_search;
-        guint          snippet_loaded : 1;
-} DhAssistantViewPriv;
+        DhLink *link;
+        gchar *current_search;
+        guint snippet_loaded : 1;
+} DhAssistantViewPrivate;
 
 enum {
         SIGNAL_OPEN_URI,
-        SIGNAL_LAST
+        N_SIGNALS
 };
-static guint signals[SIGNAL_LAST] = { 0 };
 
-G_DEFINE_TYPE (DhAssistantView, dh_assistant_view, WEBKIT_TYPE_WEB_VIEW);
+static guint signals[N_SIGNALS] = { 0 };
 
-#define GET_PRIVATE(instance) G_TYPE_INSTANCE_GET_PRIVATE \
-  (instance, DH_TYPE_ASSISTANT_VIEW, DhAssistantViewPriv)
+G_DEFINE_TYPE_WITH_PRIVATE (DhAssistantView, dh_assistant_view, WEBKIT_TYPE_WEB_VIEW);
 
 static void
 view_finalize (GObject *object)
 {
-        DhAssistantViewPriv *priv = GET_PRIVATE (object);
+        DhAssistantView *view = DH_ASSISTANT_VIEW (object);
+        DhAssistantViewPrivate *priv = dh_assistant_view_get_instance_private (view);
 
         if (priv->link) {
                 g_object_unref (priv->link);
         }
 
-        if (priv->book_manager) {
-                g_object_unref (priv->book_manager);
-        }
-
         g_free (priv->current_search);
 
         G_OBJECT_CLASS (dh_assistant_view_parent_class)->finalize (object);
@@ -67,7 +77,7 @@ assistant_decide_policy (WebKitWebView           *web_view,
                          WebKitPolicyDecision    *decision,
                          WebKitPolicyDecisionType decision_type)
 {
-        DhAssistantViewPriv            *priv;
+        DhAssistantViewPrivate         *priv;
         const gchar                    *uri;
         WebKitNavigationPolicyDecision *navigation_decision;
         WebKitNavigationAction         *navigation_action;
@@ -80,7 +90,8 @@ assistant_decide_policy (WebKitWebView           *web_view,
                 return TRUE;
         }
 
-        priv = GET_PRIVATE (web_view);
+        priv = dh_assistant_view_get_instance_private (DH_ASSISTANT_VIEW (web_view));
+
         navigation_decision = WEBKIT_NAVIGATION_POLICY_DECISION (decision);
         navigation_action = webkit_navigation_policy_decision_get_navigation_action (navigation_decision);
         navigation_type = webkit_navigation_action_get_navigation_type (navigation_action);
@@ -133,8 +144,6 @@ dh_assistant_view_class_init (DhAssistantViewClass* klass)
         widget_class->button_press_event = assistant_button_press_event;
         web_view_class->decide_policy = assistant_decide_policy;
 
-        g_type_class_add_private (klass, sizeof (DhAssistantViewPriv));
-
         /**
          * DhAssistantView::open-uri:
          * @view: the view on which the signal is emitted
@@ -157,11 +166,9 @@ dh_assistant_view_init (DhAssistantView *view)
 /**
  * dh_assistant_view_new:
  *
- * Create a new #DhAssistantView object.
- *
- * Returns: a new #DhAssistantView object
+ * Returns: (transfer floating): a new #DhAssistantView widget.
  */
-GtkWidget*
+GtkWidget *
 dh_assistant_view_new (void)
 {
         return g_object_new (DH_TYPE_ASSISTANT_VIEW, NULL);
@@ -193,18 +200,18 @@ find_in_buffer (const gchar *buffer,
 
 /**
  * dh_assistant_view_set_link:
- * @view: a #DhAssistantView object
- * @link: (nullable): a #DhLink to set or %NULL
+ * @view: a #DhAssistantView.
+ * @link: (nullable): a #DhLink to set or %NULL.
  *
  * Open @link in the assistant view, if %NULL the view will be blanked.
  *
- * Returns: %TRUE if the requested link is open, %FALSE otherwise
+ * Returns: %TRUE if the requested link is open, %FALSE otherwise.
  */
 gboolean
 dh_assistant_view_set_link (DhAssistantView *view,
                             DhLink          *link)
 {
-        DhAssistantViewPriv *priv;
+        DhAssistantViewPrivate *priv;
         gchar               *uri;
         const gchar         *anchor;
         gchar               *filename;
@@ -219,7 +226,7 @@ dh_assistant_view_set_link (DhAssistantView *view,
 
         g_return_val_if_fail (DH_IS_ASSISTANT_VIEW (view), FALSE);
 
-        priv = GET_PRIVATE (view);
+        priv = dh_assistant_view_get_instance_private (view);
 
         if (priv->link == link) {
                 return TRUE;
@@ -407,12 +414,7 @@ dh_assistant_view_set_link (DhAssistantView *view,
                 webkit_web_view_load_uri (WEBKIT_WEB_VIEW (view), "about:blank");
         }
 
-#if GLIB_CHECK_VERSION(2,21,3)
         g_mapped_file_unref (file);
-#else
-        g_mapped_file_free (file);
-#endif
-
         g_free (filename);
 
         return TRUE;
@@ -420,18 +422,19 @@ dh_assistant_view_set_link (DhAssistantView *view,
 
 /**
  * dh_assistant_view_search:
- * @view: a #DhAssistantView object
- * @str: the search query
+ * @view: a #DhAssistantView.
+ * @str: the search query.
  *
  * Search for @str in the current assistant view.
  *
- * Returns: %TRUE if @str was found, %FALSE otherwise
+ * Returns: %TRUE if @str was found, %FALSE otherwise.
  */
 gboolean
 dh_assistant_view_search (DhAssistantView *view,
                           const gchar     *str)
 {
-        DhAssistantViewPriv *priv;
+        DhAssistantViewPrivate *priv;
+        DhBookManager       *book_manager;
         const gchar         *name;
         DhLink              *link;
         DhLink              *exact_link;
@@ -441,7 +444,7 @@ dh_assistant_view_search (DhAssistantView *view,
         g_return_val_if_fail (DH_IS_ASSISTANT_VIEW (view), FALSE);
         g_return_val_if_fail (str, FALSE);
 
-        priv = GET_PRIVATE (view);
+        priv = dh_assistant_view_get_instance_private (view);
 
         /* Filter out very short strings. */
         if (strlen (str) < 4) {
@@ -457,7 +460,9 @@ dh_assistant_view_search (DhAssistantView *view,
         prefix_link = NULL;
         exact_link = NULL;
 
-        for (books = dh_book_manager_get_books (priv->book_manager);
+        book_manager = dh_book_manager_get_singleton ();
+
+        for (books = dh_book_manager_get_books (book_manager);
              !exact_link && books;
              books = g_list_next (books)) {
                 GList *l;
@@ -508,24 +513,3 @@ dh_assistant_view_search (DhAssistantView *view,
 
         return TRUE;
 }
-
-/**
- * dh_assistant_view_set_book_manager:
- * @view: a #DhAssistantView object
- * @book_manager: the new book manager
- *
- * Set a new book manager to search in.
- */
-void
-dh_assistant_view_set_book_manager (DhAssistantView *view,
-                                    DhBookManager   *book_manager)
-{
-        DhAssistantViewPriv *priv;
-
-        g_return_if_fail (DH_IS_ASSISTANT_VIEW (view));
-        g_return_if_fail (DH_IS_BOOK_MANAGER (book_manager));
-
-        priv = GET_PRIVATE (view);
-
-        priv->book_manager = g_object_ref (book_manager);
-}
