@@ -17,8 +17,8 @@
  * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __DH_KEYWORD_MODEL_H__
-#define __DH_KEYWORD_MODEL_H__
+#ifndef DH_KEYWORD_MODEL_H
+#define DH_KEYWORD_MODEL_H
 
 #include <glib-object.h>
 #include "dh-link.h"
@@ -55,8 +55,6 @@ enum {
 
 GType           dh_keyword_model_get_type  (void);
 DhKeywordModel *dh_keyword_model_new       (void);
-void            dh_keyword_model_set_words (DhKeywordModel *model,
-                                            DhBookManager  *book_manager);
 DhLink *        dh_keyword_model_filter    (DhKeywordModel *model,
                                             const gchar    *search_string,
                                             const gchar    *book_id,
@@ -64,4 +62,4 @@ DhLink *        dh_keyword_model_filter    (DhKeywordModel *model,
 
 G_END_DECLS
 
-#endif /* __DH_KEYWORD_MODEL_H__ */
+#endif /* DH_KEYWORD_MODEL_H */
