@@ -20,19 +20,32 @@
 #include "config.h"
 #include "dh-link.h"
 #include <string.h>
-#include <glib-object.h>
 #include <glib/gi18n-lib.h>
 
+/**
+ * SECTION:dh-link
+ * @Title: DhLink
+ * @Short_description: A link inside a #DhBook
+ *
+ * A #DhLink represents a link to an HTML page or somewhere inside a page (with
+ * an anchor) that is inside a #DhBook. The link can point to a specific symbol,
+ * or a page, or the top-level page of the #DhBook.
+ *
+ * A #DhLink has a type that can be retrieved with dh_link_get_link_type().
+ *
+ * There is exactly one #DhLink of type %DH_LINK_TYPE_BOOK per #DhBook object.
+ */
+
 struct _DhLink {
         /* FIXME: Those two could exist only for book to save some
          * memory.
          */
-        gchar       *id;
-        gchar       *base;
+        gchar       *book_id;
+        gchar       *base_path;
 
         gchar       *name;
         gchar       *name_collation_key;
-        gchar       *filename;
+        gchar       *relative_url;
 
         DhLink      *book;
         DhLink      *page;
@@ -49,10 +62,10 @@ G_DEFINE_BOXED_TYPE (DhLink, dh_link,
 static void
 link_free (DhLink *link)
 {
-        g_free (link->base);
-        g_free (link->id);
+        g_free (link->base_path);
+        g_free (link->book_id);
         g_free (link->name);
-        g_free (link->filename);
+        g_free (link->relative_url);
         g_free (link->name_collation_key);
 
         if (link->book) {
@@ -67,40 +80,48 @@ link_free (DhLink *link)
 
 /**
  * dh_link_new:
- * @type: the type of the content the link is pointing to
- * @base: the base path of the link
- * @id: the id of the link
- * @name: the name of the link
- * @book: (nullable): the link to the associated book or %NULL
- * @page: (nullable): the link to the associated page or %NULL
- * @filename: the filename of the link
+ * @type: the #DhLinkType.
+ * @base_path: (nullable): the base path for the book, or %NULL.
+ * @book_id: (nullable): the book ID, or %NULL.
+ * @name: the name of the link.
+ * @book: (nullable): the book that the link is contained in, or %NULL.
+ * @page: (nullable): the page that the link is contained in, or %NULL. This
+ * parameter is actually broken.
+ * @relative_url: the URL relative to the book @base_path. Can contain an
+ * anchor.
  *
- * Create a new #DhLink object.
+ * Creates a new #DhLink.
  *
- * If type is one of #DH_LINK_TYPE_BOOK or #DH_LINK_TYPE_PAGE then the @book and
- * @page links have to be provided.
+ * @base_path and @book_id must be provided only for a link of type
+ * %DH_LINK_TYPE_BOOK.
  *
- * Returns: a new #DhLink object
+ * If @type is not a #DH_LINK_TYPE_BOOK and not a #DH_LINK_TYPE_PAGE, then the
+ * @book and @page links must be provided.
+ *
+ * @name and @relative_url must always be provided.
+ *
+ * Returns: a new #DhLink.
  */
 DhLink *
 dh_link_new (DhLinkType   type,
-             const gchar *base,
-             const gchar *id,
+             const gchar *base_path,
+             const gchar *book_id,
              const gchar *name,
              DhLink      *book,
              DhLink      *page,
-             const gchar *filename)
+             const gchar *relative_url)
 {
         DhLink *link;
 
         g_return_val_if_fail (name != NULL, NULL);
-        g_return_val_if_fail (filename != NULL, NULL);
+        g_return_val_if_fail (relative_url != NULL, NULL);
 
         if (type == DH_LINK_TYPE_BOOK) {
-                g_return_val_if_fail (base != NULL, NULL);
-                g_return_val_if_fail (id != NULL, NULL);
+                g_return_val_if_fail (base_path != NULL, NULL);
+                g_return_val_if_fail (book_id != NULL, NULL);
         }
-        if (type != DH_LINK_TYPE_BOOK && type != DH_LINK_TYPE_PAGE) {
+        if (type != DH_LINK_TYPE_BOOK &&
+            type != DH_LINK_TYPE_PAGE) {
                 g_return_val_if_fail (book != NULL, NULL);
                 g_return_val_if_fail (page != NULL, NULL);
         }
@@ -111,17 +132,17 @@ dh_link_new (DhLinkType   type,
         link->type = type;
 
         if (type == DH_LINK_TYPE_BOOK) {
-                link->base = g_strdup (base);
-                link->id = g_strdup (id);
+                link->base_path = g_strdup (base_path);
+                link->book_id = g_strdup (book_id);
         }
 
         link->name = g_strdup (name);
-        link->filename = g_strdup (filename);
+        link->relative_url = g_strdup (relative_url);
 
-        if (book) {
+        if (book != NULL) {
                 link->book = dh_link_ref (book);
         }
-        if (page) {
+        if (page != NULL) {
                 link->page = dh_link_ref (page);
         }
 
@@ -130,12 +151,15 @@ dh_link_new (DhLinkType   type,
 
 /**
  * dh_link_compare:
- * @a: (type DhLink): a #DhLink object
- * @b: (type DhLink): the #DhLink to compare
+ * @a: (type DhLink): a #DhLink.
+ * @b: (type DhLink): a #DhLink.
  *
- * Compare the link @a and @b.
+ * Compares the links @a and @b. This function is used to determine in which
+ * order the links should be displayed.
  *
- * Returns: an integer less than, equal to, or greater than zero, if @a is <, == or > than @b.
+ * Returns: an integer less than zero if @a should appear before @b; zero if
+ * there are no preferences; an integer greater than zero if @b should appear
+ * before @a.
  */
 gint
 dh_link_compare (gconstpointer a,
@@ -154,9 +178,9 @@ dh_link_compare (gconstpointer a,
         }
 
         /* Collation-based sorting */
-        if (G_UNLIKELY (!la->name_collation_key))
+        if (G_UNLIKELY (la->name_collation_key == NULL))
                 la->name_collation_key = g_utf8_collate_key (la->name, -1);
-        if (G_UNLIKELY (!lb->name_collation_key))
+        if (G_UNLIKELY (lb->name_collation_key == NULL))
                 lb->name_collation_key = g_utf8_collate_key (lb->name, -1);
 
         diff = strcmp (la->name_collation_key,
@@ -183,11 +207,11 @@ dh_link_compare (gconstpointer a,
 
 /**
  * dh_link_ref:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Increase the reference count of @link.
+ * Increases the reference count of @link.
  *
- * Returns: (transfer full): the @link object
+ * Returns: (transfer full): the @link.
  */
 DhLink *
 dh_link_ref (DhLink *link)
@@ -201,9 +225,9 @@ dh_link_ref (DhLink *link)
 
 /**
  * dh_link_unref:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Decrease the reference count of @link.
+ * Decreases the reference count of @link.
  */
 void
 dh_link_unref (DhLink *link)
@@ -218,11 +242,9 @@ dh_link_unref (DhLink *link)
 
 /**
  * dh_link_get_name:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the name of the link.
- *
- * Returns: the name of the link
+ * Returns: the name of the @link.
  */
 const gchar *
 dh_link_get_name (DhLink *link)
@@ -232,16 +254,14 @@ dh_link_get_name (DhLink *link)
 
 /**
  * dh_link_get_book_name:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the book name the link is pointing to.
- *
- * Returns: the book name the link is pointing to
+ * Returns: the name of the book that the @link is contained in.
  */
 const gchar *
 dh_link_get_book_name (DhLink *link)
 {
-        if (link->book) {
+        if (link->book != NULL) {
                 return link->book->name;
         }
 
@@ -250,16 +270,16 @@ dh_link_get_book_name (DhLink *link)
 
 /**
  * dh_link_get_page_name:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the page name the link is pointing to.
- *
- * Returns: the page name the link is pointing to
+ * Returns: the name of the page that the @link is contained in.
+ * Deprecated: 3.26: This function is used nowhere and is actually broken, it
+ * returns the book name.
  */
 const gchar *
 dh_link_get_page_name (DhLink *link)
 {
-        if (link->page) {
+        if (link->page != NULL) {
                 return link->page->name;
         }
 
@@ -268,20 +288,19 @@ dh_link_get_page_name (DhLink *link)
 
 /**
  * dh_link_get_file_name:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the file name of the link.
- *
- * Returns: the file name of the link
+ * Returns: the name of the file that the @link is contained in.
  */
 const gchar *
 dh_link_get_file_name (DhLink *link)
 {
-        /* Return filename if the link is itself a page
-         * or if the link is within a page */
-        if (link->page ||
+        /* Return filename if the link is itself a page or if the link is within
+         * a page.
+         */
+        if (link->page != NULL ||
             link->type == DH_LINK_TYPE_PAGE) {
-                return link->filename;
+                return link->relative_url;
         }
 
         return "";
@@ -289,21 +308,19 @@ dh_link_get_file_name (DhLink *link)
 
 /**
  * dh_link_get_book_id:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the book id the link is pointing to.
- *
- * Returns: the book id the link is pointing to
+ * Returns: the book ID.
  */
 const gchar *
 dh_link_get_book_id (DhLink *link)
 {
         if (link->type == DH_LINK_TYPE_BOOK) {
-                return link->id;
+                return link->book_id;
         }
 
         if (link->book) {
-                return link->book->id;
+                return link->book->book_id;
         }
 
         return "";
@@ -311,33 +328,34 @@ dh_link_get_book_id (DhLink *link)
 
 /**
  * dh_link_get_uri:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the uri of the link.
- *
- * Returns: the uri of the link
+ * Returns: the @link URI. Free with g_free() when no longer needed.
  */
 gchar *
 dh_link_get_uri (DhLink *link)
 {
-        gchar *base, *filename, *uri, *anchor;
+        const gchar *base_path;
+        gchar *filename;
+        gchar *uri;
+        gchar *anchor;
 
         if (link->type == DH_LINK_TYPE_BOOK)
-                base = link->base;
+                base_path = link->base_path;
         else
-                base = link->book->base;
+                base_path = link->book->base_path;
 
-        filename = g_build_filename (base, link->filename, NULL);
+        filename = g_build_filename (base_path, link->relative_url, NULL);
 
         anchor = strrchr (filename, '#');
-        if (anchor) {
+        if (anchor != NULL) {
                 *anchor = '\0';
                 anchor++;
         }
 
         uri = g_filename_to_uri (filename, NULL, NULL);
 
-        if (anchor) {
+        if (anchor != NULL) {
                 gchar *uri_with_anchor;
 
                 uri_with_anchor = g_strconcat (uri, "#", anchor, NULL);
@@ -351,11 +369,9 @@ dh_link_get_uri (DhLink *link)
 
 /**
  * dh_link_get_link_type:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the type of the link.
- *
- * Returns: the type of the link
+ * Returns: the #DhLinkType of @link.
  */
 DhLinkType
 dh_link_get_link_type (DhLink *link)
@@ -365,11 +381,9 @@ dh_link_get_link_type (DhLink *link)
 
 /**
  * dh_link_get_flags:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the flags of the link.
- *
- * Returns: the flags of the link
+ * Returns: the #DhLinkFlags of @link.
  */
 DhLinkFlags
 dh_link_get_flags (DhLink *link)
@@ -379,10 +393,10 @@ dh_link_get_flags (DhLink *link)
 
 /**
  * dh_link_set_flags:
- * @link: a #DhLink object
- * @flags: the new flags of the link
+ * @link: a #DhLink.
+ * @flags: the new flags of the link.
  *
- * Set the flags of the link.
+ * Sets the flags of the link.
  */
 void
 dh_link_set_flags (DhLink      *link,
@@ -393,11 +407,10 @@ dh_link_set_flags (DhLink      *link,
 
 /**
  * dh_link_get_type_as_string:
- * @link: a #DhLink object
+ * @link: a #DhLink.
  *
- * Get the link type name.
- *
- * Returns: the link type name translated in the current language
+ * Returns: a string representation of the #DhLinkType of @link, translated in
+ * the current language.
  */
 const gchar *
 dh_link_get_type_as_string (DhLink *link)
