@@ -17,11 +17,28 @@
  * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __DH_LINK_H__
-#define __DH_LINK_H__
+#ifndef DH_LINK_H
+#define DH_LINK_H
 
 #include <glib-object.h>
 
+G_BEGIN_DECLS
+
+/**
+ * DhLinkType:
+ * @DH_LINK_TYPE_BOOK: The top-level page of a #DhBook.
+ * @DH_LINK_TYPE_PAGE: A page.
+ * @DH_LINK_TYPE_KEYWORD: Another kind of keyword.
+ * @DH_LINK_TYPE_FUNCTION: A function keyword.
+ * @DH_LINK_TYPE_STRUCT: A struct keyword.
+ * @DH_LINK_TYPE_MACRO: A macro keyword.
+ * @DH_LINK_TYPE_ENUM: An enum keyword.
+ * @DH_LINK_TYPE_TYPEDEF: A typedef keyword.
+ * @DH_LINK_TYPE_PROPERTY: A property keyword.
+ * @DH_LINK_TYPE_SIGNAL: A signal keyword.
+ *
+ * The type of the content the link points to.
+ */
 typedef enum {
         DH_LINK_TYPE_BOOK,
         DH_LINK_TYPE_PAGE,
@@ -30,9 +47,16 @@ typedef enum {
         DH_LINK_TYPE_STRUCT,
         DH_LINK_TYPE_MACRO,
         DH_LINK_TYPE_ENUM,
-        DH_LINK_TYPE_TYPEDEF
+        DH_LINK_TYPE_TYPEDEF,
+        DH_LINK_TYPE_PROPERTY,
+        DH_LINK_TYPE_SIGNAL
 } DhLinkType;
 
+/**
+ * DhLinkFlags:
+ * @DH_LINK_FLAGS_NONE: No flags set.
+ * @DH_LINK_FLAGS_DEPRECATED: The symbol that the link points to is deprecated.
+ */
 typedef enum {
         DH_LINK_FLAGS_NONE       = 0,
         DH_LINK_FLAGS_DEPRECATED = 1 << 0
@@ -44,20 +68,18 @@ typedef struct _DhLink DhLink;
 
 GType        dh_link_get_type           (void);
 DhLink *     dh_link_new                (DhLinkType     type,
-                                         const gchar   *base,
-                                         const gchar   *id,
+                                         const gchar   *base_path,
+                                         const gchar   *book_id,
                                          const gchar   *name,
                                          DhLink        *book,
                                          DhLink        *page,
-                                         const gchar   *filename);
-void         dh_link_free               (DhLink        *link);
-gint         dh_link_compare            (gconstpointer  a,
-                                         gconstpointer  b);
+                                         const gchar   *relative_url);
 DhLink *     dh_link_ref                (DhLink        *link);
 void         dh_link_unref              (DhLink        *link);
+gint         dh_link_compare            (gconstpointer  a,
+                                         gconstpointer  b);
 const gchar *dh_link_get_name           (DhLink        *link);
 const gchar *dh_link_get_book_name      (DhLink        *link);
-const gchar *dh_link_get_page_name      (DhLink        *link);
 const gchar *dh_link_get_file_name      (DhLink        *link);
 const gchar *dh_link_get_book_id        (DhLink        *link);
 gchar       *dh_link_get_uri            (DhLink        *link);
@@ -67,4 +89,9 @@ void         dh_link_set_flags          (DhLink        *link,
 DhLinkType   dh_link_get_link_type      (DhLink        *link);
 const gchar *dh_link_get_type_as_string (DhLink        *link);
 
-#endif /* __DH_LINK_H__ */
+G_DEPRECATED
+const gchar *dh_link_get_page_name      (DhLink        *link);
+
+G_END_DECLS
+
+#endif /* DH_LINK_H */
