@@ -30,9 +30,32 @@
 #include "dh-book.h"
 #include "dh-book-tree.h"
 
-typedef struct {
-        DhBookManager           *book_manager;
+/**
+ * SECTION:dh-sidebar
+ * @Title: DhSidebar
+ * @Short_description: The sidebar
+ *
+ * In the Devhelp application, there is one #DhSidebar per main window,
+ * displayed in the left side panel.
+ *
+ * A #DhSidebar contains:
+ * - a #GtkSearchEntry at the top;
+ * - a #DhBookTree (a subclass of #GtkTreeView);
+ * - another #GtkTreeView (displaying a list, not a tree) with a #DhKeywordModel
+ *   as its model.
+ *
+ * When the #GtkSearchEntry is empty, the #DhBookTree is shown. When the
+ * #GtkSearchEntry is not empty, it shows the search results in the other
+ * #GtkTreeView. The two #GtkTreeView's cannot be both visible at the same time,
+ * it's either one or the other.
+ *
+ * The #DhSidebar::link-selected signal is emitted when one element in one of
+ * the #GtkTreeView's is selected. When that happens, the Devhelp application
+ * opens the link in a #WebKitWebView shown at the right side of the main
+ * window.
+ */
 
+typedef struct {
         DhBookTree              *book_tree;
         GtkScrolledWindow       *sw_book_tree;
 
@@ -46,17 +69,12 @@ typedef struct {
         guint                    idle_filter_id;
 } DhSidebarPrivate;
 
-enum {
-        PROP_0,
-        PROP_BOOK_MANAGER
-};
-
 enum {
         LINK_SELECTED,
-        LAST_SIGNAL
+        N_SIGNALS
 };
 
-static gint signals[LAST_SIGNAL] = { 0 };
+static guint signals[N_SIGNALS] = { 0 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (DhSidebar, dh_sidebar, GTK_TYPE_BOX)
 
@@ -167,11 +185,12 @@ sidebar_book_deleted_or_disabled_cb (DhBookManager *book_manager,
 static void
 sidebar_completion_populate (DhSidebar *sidebar)
 {
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
+        DhBookManager *book_manager;
         GList *books;
         GList *l;
 
-        books = dh_book_manager_get_books (priv->book_manager);
+        book_manager = dh_book_manager_get_singleton ();
+        books = dh_book_manager_get_books (book_manager);
 
         for (l = books; l != NULL; l = l->next)
                 sidebar_completion_add_book (sidebar, DH_BOOK (l->data));
@@ -340,10 +359,8 @@ sidebar_entry_insert_text_cb (GtkEntry    *entry,
 
 /**
  * dh_sidebar_set_search_string:
- * @sidebar: a #DhSidebar object
- * @str: the string to seach
- *
- * Set the search string to @str.
+ * @sidebar: a #DhSidebar.
+ * @str: the string to search.
  */
 void
 dh_sidebar_set_search_string (DhSidebar   *sidebar,
@@ -358,13 +375,23 @@ dh_sidebar_set_search_string (DhSidebar   *sidebar,
         gtk_entry_set_text (priv->entry, str);
         gtk_editable_set_position (GTK_EDITABLE (priv->entry), -1);
         gtk_editable_select_region (GTK_EDITABLE (priv->entry), -1, -1);
+
+        /* If the GtkEntry text was already equal to @str, the
+         * GtkEditable::changed signal was not emitted, so force to emit it to
+         * call sidebar_entry_changed_cb(), forcing a new search. If an exact
+         * match is found, the DhSidebar::link-selected signal will be emitted,
+         * to re-jump to that symbol (even if the GtkEntry text was equal, it
+         * doesn't mean that the WebKitWebView was showing the exact match).
+         * https://bugzilla.gnome.org/show_bug.cgi?id=776596
+         */
+        g_signal_emit_by_name (priv->entry, "changed");
 }
 
 /**
  * dh_sidebar_set_search_focus:
- * @sidebar: a #DhSidebar object
+ * @sidebar: a #DhSidebar.
  *
- * Give the focus to the search entry.
+ * Gives the focus to the search entry.
  */
 void
 dh_sidebar_set_search_focus (DhSidebar *sidebar)
@@ -436,11 +463,10 @@ sidebar_book_tree_link_selected_cb (DhBookTree *book_tree,
 
 /**
  * dh_sidebar_get_selected_book:
- * @sidebar: a #DhSidebar object
+ * @sidebar: a #DhSidebar.
  *
- * Get the link pointing to the selected book.
- *
- * Returns: (nullable) (transfer full): the selected book link
+ * Returns: (nullable) (transfer none): the #DhLink of the selected book, or
+ * %NULL if there is no selection.
  */
 DhLink *
 dh_sidebar_get_selected_book (DhSidebar *sidebar)
@@ -456,10 +482,8 @@ dh_sidebar_get_selected_book (DhSidebar *sidebar)
 
 /**
  * dh_sidebar_select_uri:
- * @sidebar: a #DhSidebar object
- * @uri: the uri to select
- *
- * Select the given @uri.
+ * @sidebar: a #DhSidebar.
+ * @uri: the URI to select.
  */
 void
 dh_sidebar_select_uri (DhSidebar   *sidebar,
@@ -476,18 +500,16 @@ dh_sidebar_select_uri (DhSidebar   *sidebar,
 
 /**
  * dh_sidebar_new:
- * @book_manager: the book manager
+ * @book_manager: (nullable): a #DhBookManager. This parameter is deprecated,
+ * you should just pass %NULL.
  *
- * Create a new #DhSidebar object.
- *
- * Returns: a new #DhSidebar object
+ * Returns: (transfer floating): a new #DhSidebar widget.
  */
 GtkWidget *
 dh_sidebar_new (DhBookManager *book_manager)
 {
         return GTK_WIDGET (g_object_new (DH_TYPE_SIDEBAR,
                                          "orientation", GTK_ORIENTATION_VERTICAL,
-                                         "book-manager", book_manager,
                                          NULL));
 }
 
@@ -496,7 +518,6 @@ dh_sidebar_dispose (GObject *object)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
 
-        g_clear_object (&priv->book_manager);
         g_clear_object (&priv->hitlist_model);
 
         if (priv->idle_complete_id != 0) {
@@ -526,7 +547,8 @@ static void
 dh_sidebar_init (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv;
-        GtkCellRenderer  *cell;
+        GtkCellRenderer *cell;
+        DhBookManager *book_manager;
 
         priv = dh_sidebar_get_instance_private (sidebar);
 
@@ -587,6 +609,33 @@ dh_sidebar_init (DhSidebar *sidebar)
                            GTK_WIDGET (priv->hitlist_view));
         gtk_box_pack_start (GTK_BOX (sidebar), GTK_WIDGET (priv->sw_hitlist), TRUE, TRUE, 0);
 
+        /* Setup book manager */
+        book_manager = dh_book_manager_get_singleton ();
+
+        g_signal_connect_object (book_manager,
+                                 "book-created",
+                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
+                                 sidebar,
+                                 0);
+
+        g_signal_connect_object (book_manager,
+                                 "book-enabled",
+                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
+                                 sidebar,
+                                 0);
+
+        g_signal_connect_object (book_manager,
+                                 "book-deleted",
+                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
+                                 sidebar,
+                                 0);
+
+        g_signal_connect_object (book_manager,
+                                 "book-disabled",
+                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
+                                 sidebar,
+                                 0);
+
         /* Setup the book tree */
         priv->sw_book_tree = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
         gtk_widget_show (GTK_WIDGET (priv->sw_book_tree));
@@ -595,78 +644,7 @@ dh_sidebar_init (DhSidebar *sidebar)
                                         GTK_POLICY_NEVER,
                                         GTK_POLICY_AUTOMATIC);
 
-        gtk_widget_show_all (GTK_WIDGET (sidebar));
-}
-
-static void
-dh_sidebar_get_property (GObject    *object,
-                         guint       prop_id,
-                         GValue     *value,
-                         GParamSpec *pspec)
-{
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
-
-        switch (prop_id) {
-                case PROP_BOOK_MANAGER:
-                        g_value_set_object (value, priv->book_manager);
-                        break;
-                default:
-                        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                        break;
-        }
-}
-
-static void
-dh_sidebar_set_property (GObject      *object,
-                         guint         prop_id,
-                         const GValue *value,
-                         GParamSpec   *pspec)
-{
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
-
-        switch (prop_id) {
-                case PROP_BOOK_MANAGER:
-                        g_return_if_fail (priv->book_manager == NULL);
-                        priv->book_manager = g_value_dup_object (value);
-                        break;
-                default:
-                        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                        break;
-        }
-}
-
-static void
-dh_sidebar_constructed (GObject *object)
-{
-        DhSidebar *sidebar = DH_SIDEBAR (object);
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-
-        /* Setup book manager */
-        g_signal_connect_object (priv->book_manager,
-                                 "book-created",
-                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
-                                 sidebar,
-                                 0);
-
-        g_signal_connect_object (priv->book_manager,
-                                 "book-enabled",
-                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
-                                 sidebar,
-                                 0);
-
-        g_signal_connect_object (priv->book_manager,
-                                 "book-deleted",
-                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
-                                 sidebar,
-                                 0);
-
-        g_signal_connect_object (priv->book_manager,
-                                 "book-disabled",
-                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
-                                 sidebar,
-                                 0);
-
-        priv->book_tree = DH_BOOK_TREE (dh_book_tree_new (priv->book_manager));
+        priv->book_tree = dh_book_tree_new ();
         gtk_widget_show (GTK_WIDGET (priv->book_tree));
         g_signal_connect (priv->book_tree,
                           "link-selected",
@@ -677,9 +655,7 @@ dh_sidebar_constructed (GObject *object)
 
         sidebar_completion_populate (sidebar);
 
-        dh_keyword_model_set_words (priv->hitlist_model, priv->book_manager);
-
-        G_OBJECT_CLASS (dh_sidebar_parent_class)->constructed (object);
+        gtk_widget_show_all (GTK_WIDGET (sidebar));
 }
 
 static void
@@ -689,31 +665,18 @@ dh_sidebar_class_init (DhSidebarClass *klass)
 
         object_class->dispose = dh_sidebar_dispose;
         object_class->finalize = dh_sidebar_finalize;
-        object_class->get_property = dh_sidebar_get_property;
-        object_class->set_property = dh_sidebar_set_property;
-        object_class->constructed = dh_sidebar_constructed;
-
-        g_object_class_install_property (object_class,
-                                         PROP_BOOK_MANAGER,
-                                         g_param_spec_object ("book-manager",
-                                                              "Book Manager",
-                                                              "The book maanger",
-                                                              DH_TYPE_BOOK_MANAGER,
-                                                              G_PARAM_READWRITE |
-                                                              G_PARAM_CONSTRUCT_ONLY));
 
         /**
          * DhSidebar::link-selected:
-         * @sidebar: a #DhSidebar object
-         * @link: the selected #DhLink
+         * @sidebar: a #DhSidebar.
+         * @link: the selected #DhLink.
          */
         signals[LINK_SELECTED] =
-                g_signal_new ("link_selected",
+                g_signal_new ("link-selected",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (DhSidebarClass, link_selected),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__POINTER,
+                              NULL, NULL, NULL,
                               G_TYPE_NONE,
-                              1, G_TYPE_POINTER);
+                              1, DH_TYPE_LINK);
 }
