@@ -31,8 +31,8 @@
 #include "dh-book.h"
 #include "dh-sidebar.h"
 #include "dh-util.h"
-#include "dh-enum-types.h"
 #include "dh-settings.h"
+#include "dh-enum-types-app.h"
 
 typedef struct {
         GMenuModel     *gear_app_menu;
@@ -48,17 +48,16 @@ typedef struct {
         GtkButton      *go_down_button;
 
         DhLink         *selected_search_link;
-        DhSettings     *settings;
         GtkSettings    *gtk_settings;
         gulong          xft_dpi_changed_id;
 } DhWindowPrivate;
 
 enum {
         OPEN_LINK,
-        LAST_SIGNAL
+        N_SIGNALS
 };
 
-static gint signals[LAST_SIGNAL] = { 0 };
+static guint signals[N_SIGNALS] = { 0 };
 
 static guint tab_accel_keys[] = {
         GDK_KEY_1, GDK_KEY_2, GDK_KEY_3, GDK_KEY_4, GDK_KEY_5,
@@ -93,8 +92,6 @@ static const guint n_zoom_levels = G_N_ELEMENTS (zoom_levels);
 static void           window_search_link_selected_cb (GObject         *ignored,
                                                       DhLink          *link,
                                                       DhWindow        *window);
-static void           window_check_history           (DhWindow        *window,
-                                                      WebKitWebView   *web_view);
 static void           window_web_view_tab_accel_cb   (GtkAccelGroup   *accel_group,
                                                       GObject         *object,
                                                       guint            key,
@@ -348,6 +345,24 @@ window_update_zoom_actions_state (DhWindow *window)
         g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
 }
 
+static void
+window_update_back_forward_actions_state (DhWindow *window)
+{
+        WebKitWebView *web_view;
+        GAction *action;
+        gboolean enabled;
+
+        web_view = window_get_active_web_view (window);
+
+        enabled = web_view != NULL ? webkit_web_view_can_go_back (web_view) : FALSE;
+        action = g_action_map_lookup_action (G_ACTION_MAP (window), "go-back");
+        g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
+
+        enabled = web_view != NULL ? webkit_web_view_can_go_forward (web_view) : FALSE;
+        action = g_action_map_lookup_action (G_ACTION_MAP (window), "go-forward");
+        g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
+}
+
 static void
 show_sidebar_cb (GSimpleAction *action,
                  GVariant      *parameter,
@@ -560,13 +575,17 @@ static void
 update_fonts_on_dpi_change (DhWindow *window)
 {
         DhWindowPrivate *priv;
+        DhSettings *settings;
         gchar *font_fixed = NULL;
         gchar *font_variable = NULL;
         WebKitWebView *view;
         gint i;
 
         priv = dh_window_get_instance_private (window);
-        dh_settings_get_selected_fonts (priv->settings, &font_fixed, &font_variable);
+
+        settings = dh_settings_get_singleton ();
+        dh_settings_get_selected_fonts (settings, &font_fixed, &font_variable);
+
         if (font_fixed != NULL && font_variable != NULL) {
                 /* change font for all pages */
                 for (i = 0; i < gtk_notebook_get_n_pages (priv->notebook); i++) {
@@ -636,12 +655,11 @@ static gboolean
 dh_window_configure_event (GtkWidget         *window,
                            GdkEventConfigure *event)
 {
-        DhWindowPrivate *priv;
-
-        priv = dh_window_get_instance_private (DH_WINDOW (window));
+        DhSettings *settings;
 
+        settings = dh_settings_get_singleton ();
         dh_util_window_settings_save (GTK_WINDOW (window),
-                                      dh_settings_peek_window_settings (priv->settings),
+                                      dh_settings_peek_window_settings (settings),
                                       TRUE);
 
         if (GTK_WIDGET_CLASS (dh_window_parent_class)->configure_event == NULL)
@@ -653,7 +671,8 @@ dh_window_configure_event (GtkWidget         *window,
 static void
 dh_window_init (DhWindow *window)
 {
-        DhWindowPrivate  *priv;
+        DhWindowPrivate *priv;
+        DhSettings    *settings;
         GtkAccelGroup *accel_group;
         GClosure      *closure;
         guint          i;
@@ -668,8 +687,8 @@ dh_window_init (DhWindow *window)
         }
 
         /* handle settings */
-        priv->settings = dh_settings_get ();
-        g_signal_connect_object (priv->settings,
+        settings = dh_settings_get_singleton ();
+        g_signal_connect_object (settings,
                                  "fonts-changed",
                                  G_CALLBACK (settings_fonts_changed_cb),
                                  window,
@@ -715,8 +734,6 @@ dh_window_dispose (GObject *object)
                 priv->xft_dpi_changed_id = 0;
         }
 
-        g_clear_object (&priv->settings);
-
         G_OBJECT_CLASS (dh_window_parent_class)->dispose (object);
 }
 
@@ -733,19 +750,12 @@ dh_window_class_init (DhWindowClass *klass)
 
         klass->open_link = dh_window_open_link;
 
-        /**
-         * DhWindow::open-link:
-         * @window: a #DhWindow object
-         * @location: the location of the link
-         * @flags: a #DhOpenLinkFlags
-         */
         signals[OPEN_LINK] =
                 g_signal_new ("open-link",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (DhWindowClass, open_link),
-                              NULL, NULL,
-                              g_cclosure_marshal_generic,
+                              NULL, NULL, NULL,
                               G_TYPE_NONE,
                               2,
                               G_TYPE_STRING,
@@ -790,14 +800,11 @@ window_web_view_switch_page_cb (GtkNotebook *notebook,
                 if (location != NULL)
                         dh_sidebar_select_uri (priv->sidebar, location);
 
-                window_check_history (window, new_web_view);
-
                 window_update_title (window, new_web_view, NULL);
         } else {
                 /* i18n: Please don't translate "Devhelp" (it's marked as translatable
                  * for transliteration only) */
                 gtk_window_set_title (GTK_WINDOW (window), _("Devhelp"));
-                window_check_history (window, NULL);
         }
 }
 
@@ -808,19 +815,16 @@ window_web_view_switch_page_after_cb (GtkNotebook *notebook,
                                       DhWindow    *window)
 {
         window_update_zoom_actions_state (window);
+        window_update_back_forward_actions_state (window);
 }
 
 static void
 window_populate (DhWindow *window)
 {
-        DhWindowPrivate *priv;
-        DhBookManager *book_manager;
-
-        priv = dh_window_get_instance_private (window);
-        book_manager = dh_app_peek_book_manager (DH_APP (gtk_window_get_application (GTK_WINDOW (window))));
+        DhWindowPrivate *priv = dh_window_get_instance_private (window);
 
         /* Sidebar */
-        priv->sidebar = DH_SIDEBAR (dh_sidebar_new (book_manager));
+        priv->sidebar = DH_SIDEBAR (dh_sidebar_new (NULL));
         gtk_widget_set_vexpand (GTK_WIDGET (priv->sidebar), TRUE);
         gtk_widget_set_hexpand (GTK_WIDGET (priv->sidebar), TRUE);
         gtk_widget_show (GTK_WIDGET (priv->sidebar));
@@ -905,7 +909,7 @@ find_library_equivalent (DhWindow    *window,
                 return NULL;
         }
 
-        book_manager = dh_app_peek_book_manager (DH_APP (gtk_window_get_application (GTK_WINDOW (window))));
+        book_manager = dh_book_manager_get_singleton ();
 
         for (books = dh_book_manager_get_books (book_manager);
              local_uri == NULL && books != NULL;
@@ -980,7 +984,7 @@ window_web_view_decide_policy_cb (WebKitWebView            *web_view,
 
         if (!g_str_has_prefix (uri, "file://")) {
                 webkit_policy_decision_ignore (policy_decision);
-                gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, NULL);
+                gtk_show_uri_on_window (GTK_WINDOW (window), uri, GDK_CURRENT_TIME, NULL);
                 return GDK_EVENT_STOP;
         }
 
@@ -1002,7 +1006,6 @@ window_web_view_load_changed_cb (WebKitWebView   *web_view,
 
         uri = webkit_web_view_get_uri (web_view);
         dh_sidebar_select_uri (priv->sidebar, uri);
-        window_check_history (window, web_view);
 }
 
 static gboolean
@@ -1118,24 +1121,6 @@ window_search_link_selected_cb (GObject  *ignored,
         uri = dh_link_get_uri (link);
         webkit_web_view_load_uri (view, uri);
         g_free (uri);
-
-        window_check_history (window, view);
-}
-
-static void
-window_check_history (DhWindow      *window,
-                      WebKitWebView *web_view)
-{
-        GAction *action;
-        gboolean enabled;
-
-        enabled = web_view != NULL ? webkit_web_view_can_go_forward (web_view) : FALSE;
-        action = g_action_map_lookup_action (G_ACTION_MAP (window), "go-forward");
-        g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-
-        enabled = web_view != NULL ? webkit_web_view_can_go_back (web_view) : FALSE;
-        action = g_action_map_lookup_action (G_ACTION_MAP (window), "go-back");
-        g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
 }
 
 static void
@@ -1315,6 +1300,7 @@ window_open_new_tab (DhWindow    *window,
                      gboolean     switch_focus)
 {
         DhWindowPrivate *priv;
+        DhSettings   *settings;
         GtkWidget    *view;
         GtkWidget    *vbox;
         GtkWidget    *label;
@@ -1322,6 +1308,7 @@ window_open_new_tab (DhWindow    *window,
         GtkWidget    *info_bar;
         gchar *font_fixed = NULL;
         gchar *font_variable = NULL;
+        WebKitBackForwardList *back_forward_list;
 
         priv = dh_window_get_instance_private (window);
 
@@ -1329,8 +1316,10 @@ window_open_new_tab (DhWindow    *window,
         view = webkit_web_view_new ();
         apply_webview_settings (WEBKIT_WEB_VIEW (view));
         gtk_widget_show (view);
+
         /* get the current fonts and set them on the new view */
-        dh_settings_get_selected_fonts (priv->settings, &font_fixed, &font_variable);
+        settings = dh_settings_get_singleton ();
+        dh_settings_get_selected_fonts (settings, &font_fixed, &font_variable);
         dh_util_view_set_font (WEBKIT_WEB_VIEW (view), font_fixed, font_variable);
         g_free (font_fixed);
         g_free (font_variable);
@@ -1379,6 +1368,13 @@ window_open_new_tab (DhWindow    *window,
                           G_CALLBACK (window_web_view_scroll_event_cb),
                           window);
 
+        back_forward_list = webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view));
+        g_signal_connect_object (back_forward_list,
+                                 "changed",
+                                 G_CALLBACK (window_update_back_forward_actions_state),
+                                 window,
+                                 G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+
         num = gtk_notebook_append_page (priv->notebook, vbox, NULL);
         gtk_container_child_set (GTK_CONTAINER (priv->notebook), vbox,
                                  "tab-expand", TRUE,
@@ -1556,19 +1552,12 @@ window_tab_set_title (DhWindow      *window,
         }
 }
 
-/**
- * dh_window_new:
- * @application: the #DhApp owning this window
- *
- * Create a new #DhWindow object.
- *
- * Returns: a new #DhWindow object
- */
 GtkWidget *
 dh_window_new (DhApp *application)
 {
-        DhWindow     *window;
+        DhWindow *window;
         DhWindowPrivate *priv;
+        DhSettings *settings;
 
         window = g_object_new (DH_TYPE_WINDOW,
                                "application", application,
@@ -1578,24 +1567,18 @@ dh_window_new (DhApp *application)
 
         window_populate (window);
 
+        settings = dh_settings_get_singleton ();
         dh_util_window_settings_restore (GTK_WINDOW (window),
-                                         dh_settings_peek_window_settings (priv->settings),
+                                         dh_settings_peek_window_settings (settings),
                                          TRUE);
 
-        g_settings_bind (dh_settings_peek_paned_settings (priv->settings), "position",
+        g_settings_bind (dh_settings_peek_paned_settings (settings), "position",
                          priv->hpaned, "position",
                          G_SETTINGS_BIND_DEFAULT);
 
         return GTK_WIDGET (window);
 }
 
-/**
- * dh_window_search:
- * @window: a #DhWindow object
- * @str: the string to search
- *
- * Search for @str in the current window.
- */
 void
 dh_window_search (DhWindow    *window,
                   const gchar *str)
@@ -1609,15 +1592,7 @@ dh_window_search (DhWindow    *window,
         dh_sidebar_set_search_string (priv->sidebar, str);
 }
 
-/*< private >
- * _dh_window_display_uri:
- * @window: a #DhWindow object
- * @uri: the uri to display
- *
- * Open the @uri in the current window.
- *
- * Only call this with a URI that is known to be in the docs.
- */
+/* Only call this with a URI that is known to be in the docs. */
 void
 _dh_window_display_uri (DhWindow    *window,
                         const gchar *uri)
